#!/bin/sh

# download and package vivaldi
# 2nd version

[ $(id -u) -ne 0 ] && exec sudo -A ${0} ${@}

export TEXTDOMAIN=install_vivaldi_gui.sh
export OUTPUT_CHARSET=UTF-8

CWD=`pwd`

# func strings
IMSG=$(gettext 'Please wait... checking latest versions')
ERR_TITLE=$(gettext 'Install Vivaldi - message')
EX=$(gettext ' already exists in /mnt/home. Load it or move it! Exiting.')
TTLC=$(gettext ' needs GTK 3.0. Quit this program and install it from the package manager.')
ERR_CON=$(gettext "There seems to be a problem connecting to the Vivaldi servers.")
IS_RET=$(gettext 'is already retrieved')
ESFS=$(gettext 'There was a problem loading ')
OKSFS1=$(gettext ' is located in ')
OKSFS2=$(gettext '. You can choose to save this for later if you wish. It will be loaded when you close this window.')
SMSG=$(gettext ' is successfully installed! It is recommended that you run this program once a week to keep vivaldi updated.')
SMSG_NOINST=$(gettext ' is successfully downloaded and packaged however you did not install it. Save it or install with sfs_load. It is located in ')
ARMSG=$(gettext 'The ar utility is not installed. We will attempt to use dpkg-deb.')
DPMSG=$(gettext 'The dpkg-deb utility is not installed. Exiting')
WMSG=$(gettext 'Please wait while the package archive is unpacked...')
MSG1=$(gettext "You can download the Vivaldi stable or testing browser and create and install a squash filesystem (SFS) Puppy package.")
MSG2=$(gettext "Just hit the button of the Vivaldi version you wish to install and the download will start")
MSG3=$(gettext "Choose if you want to run as 'root' or unprivileged user 'spot'")
MEDIA=$(gettext 'Install proprietary media codec')
FMESG=$(gettext 'Installing proprietary libffmpeg.so')
FMESG_INSTALLED=$(gettext 'Proprietary libffmpeg.so is already installed for')
	
. /etc/rc.d/PUPSTATE

# gui for messages
init_msg() {
	MSG="$1" 
	[ -z "$2" ] && GTKICON=no || GTKICON=$2
export ERR='<window title="'$ERR_TITLE'" icon-name="gtk-'${GTKICON}'" resizable="false">
	<vbox>
	'"`/usr/lib/gtkdialog/xml_info fixed /usr/share/pixmaps/dl_vivaldi.svg 60 "$MSG"`"' 
	  <frame>
	    <hbox homogeneous="true">
			<button ok></button>
		</hbox>
	  </frame>
	</vbox>
</window>'
. /usr/lib/gtkdialog/xml_info gtk
gtkdialog -p ERR -c --styles=/tmp/gtkrc_xml_info.css
[ "$2" = 'yes' ] || exit 1
}

# check if an sfs is loaded
is_sfs_loaded() {
	local B_SFS_LOADED
	[ -f /tmp/sfs_load/loop_device_list ] && B_SFS_LOADED=`grep "${1}" /tmp/sfs_load/loop_device_list`
	if [ -z "$B_SFS_LOADED" ]; then
		return 1 #not loaded
	fi
	return 0
}


# check if sfs is in /mnt/home
check_sfs_exists() {
	BRS_FILE="$1"
	is_sfs_loaded $BRS_FILE && return 0
	if [ -f "/initrd/${PUP_HOME}/${BRS_FILE}" ];then
		#kill -9 $apid
		init_msg "$BRS_FILE $EX"
	fi
}

# unload
unload_sfs() {
	sfs_load
}

# download, extract and build funtions
func_dld() {
	( 
	cd $TMP
	case $1 in
		0)urxvt -bg  green -fg blue --geometry 80x20+1+1 -e ${TMP}/install-vivaldi.sh -r -f;; # stable
		1)urxvt -bg orange -fg blue --geometry 80x20+1+1 -e ${TMP}/install-vivaldi.sh -r;; # testing
	esac
	)
}

func_extract() {
	(
	cd $TMP
	if [ -z "$ARNOT" ];then
		ar p "$1" data.tar.xz > data.tar.xz
		tar xJf data.tar.xz -C $TMPDIR
	else
		dpkg-deb -x "$1" $TMPDIR
	fi
	)
}

func_mk_sfs() {
	urxvt -bg  green -fg blue --geometry 80x20+1+1 -e mksquashfs $TMPDIR "$1"
}

load_sfs() {
	sfs_load +"${1}"
	return $?
}

install_prop_media() {
	echo "downloading ffmpeg"
	# grok out latest ffmpeg URL from the just downloaded vivaldi launch script
	VIVALDI_FF_SCRIPT=${TMPDIR}/opt/${VIVALDI_STREAM_SHORT}/update-ffmpeg
	FFMPEGVER=`grep '^FFMPEG_VERSION=' $VIVALDI_FF_SCRIPT | cut -d '=' -f 2 | tr -d "'"`
	case $arch in
		i?86)
		FFMPEG_URL=`grep -m3 'FFMPEG_URL' $VIVALDI_FF_SCRIPT | grep -m1 'i386' | grep -o 'https.*\.deb'`
		;;
		x86_64|amd64)
		FFMPEG_URL=`grep -m3 'FFMPEG_URL' $VIVALDI_FF_SCRIPT | grep -m1 'amd64' | grep -o 'https.*\.deb'`
		;;
		arm*)
		FFMPEG_URL=`grep -m3 'FFMPEG_URL' $VIVALDI_FF_SCRIPT | grep -m1 'armhf' | grep -o 'https.*\.deb'`
		;;
	esac
	FFMPEG_URL=`echo ${FFMPEG_URL} | sed "s%_.*_%_${FFMPEGVER}_%"`
	PPKG=${FFMPEG_URL##*/}
	FFMPEG_INSTALL_DIR="var/opt/vivaldi/media-codecs-${FFMPEGVER%-*}"
	# download and install
	(
	cd $TMP
	urxvt -bg  green -fg blue --geometry 80x20+1+1 -e wget $FFMPEG_URL
	gtkdialog-splash -close never -bg CornflowerBlue -text "$FMSG" &
	fpid=$!
	if [ -z "$ARNOT" ];then
		ar p "$PPKG" data.tar.xz > data.tar.xz
		tar xJf data.tar.xz -C $TMPDIR
	else
		dpkg-deb -x "$PPKG" $TMPDIR
	fi
		(
		cd $TMPDIR
		FFMPEGSO=`find usr/ -type f -name 'libffmpeg.so'`
		mkdir -p ${FFMPEG_INSTALL_DIR}
		cp -a $FFMPEGSO ${FFMPEG_INSTALL_DIR}
		rm -rf usr/lib/
		)
	kill -9 $fpid
	)
}

# trap - clean up
_trap_exit() {
	cd $CWD
	rm -rf $TMP/data.tar.xz $TMP/*.deb $TMPDIR
	killall yaf-splash >/dev/null 2>&1
	echo killed
}
trap _trap_exit EXIT

# dependencies
GTK3=''
find /usr -maxdepth 4 -type d -name 'gtk-*'|grep 'lib'|grep -q '3' && GTK3='yes'
[ "$GTK3" != 'yes' ] && init_msg "Vivaldi $TTLC" 

if ! type ar >/dev/null 2>&1 ; then
	init_msg "$ARMSG" yes
	ARNOT=1
	if ! type dpkg-deb >/dev/null 2>&1 ; then
		init_msg "$DPMSG"
	fi
fi

# check arch
arch=`uname -m`

case $arch in
	arm*)DEBARCH=armhf
		CARCH=$DEBARCH;;
	i?86)DEBARCH=i386
		CARCH=$DEBARCH;;
	x86_64)DEBARCH=amd64
		CARCH=$DEBARCH;;
	*)exit;;
esac
VIVALDI=true
SVIVALDI=true

# find icons
SFSICON=/usr/share/pixmaps/puppy/package_sfs.svg
ICONTHEME=`grep -i 'icon-theme' ~/.gtkrc-2.0|sed -e 's/^gtk-icon-theme-name = //' -e 's/\"//g'`
[ -e "/usr/share/icons/$ICONTHEME/48/mimetypes/application-x-squashfs-image.svg" ] &&\
	SFSICON="/usr/share/icons/$ICONTHEME/48/mimetypes/application-x-squashfs-image.svg"
	
# tmp - working directory -if small RAM we do it on disk
TMP=${TMP:-/tmp}
read  a mem z <<< `busybox free|grep '^Mem'`
[ $mem -lt 1048576 ] && TMP=/initrd${PUP_HOME}
TMPDIR=${TMP}/vivaldi$$
mkdir -p $TMPDIR

# check if installed ####
# but wait..
gtkdialog-splash -close never -bg CornflowerBlue -text "$IMSG" &
export apid=$!
sleep 3
read zz CUR_STABLE_VERSION yy <<< $(vivaldi-stable --version 2>/dev/null | grep -E 'Vivaldi [0-9]')
NEW_STABLE_VERSION=$(wget -qO- http://repo.vivaldi.com/archive/deb/dists/stable/main/binary-$DEBARCH/Packages.gz | gzip -d | grep -A6 -x "Package: vivaldi-stable" | sed -n "/Version/s/.* //p" | sort -V | tail -n 1)
GET_STABLE_VERSION=$NEW_STABLE_VERSION
if echo $NEW_STABLE_VERSION | grep -Fq '-'; then
  VIVALDI_STABLE_REVISION=${NEW_STABLE_VERSION#*-}
  NEW_STABLE_VERSION=${NEW_STABLE_VERSION%-*}
else
  VIVALDI_STABLE_REVISION=""
fi

read xx CUR_TEST_VERSION yy <<< $(vivaldi-snapshot --version 2>/dev/null | grep -E 'Vivaldi [0-9]')
NEW_TESTING_VERSION=$(wget -qO- http://repo.vivaldi.com/archive/deb/dists/stable/main/binary-$DEBARCH/Packages.gz | gzip -d | grep -A6 -x "Package: vivaldi-snapshot" | sed -n "/Version/s/.* //p" | sort -V | tail -n 1)
GET_TESTING_VERSION=$NEW_TESTING_VERSION
if echo $NEW_TESTING_VERSION | grep -Fq '-'; then
  VIVALDI_TESTING_REVISION=${NEW_TESTING_VERSION#*-}
  NEW_TESTING_VERSION=${NEW_TESTING_VERSION%-*}
else
  VIVALDI_TESTING_REVISION=""
fi

[ -z "$NEW_STABLE_VERSION" -o -z "$NEW_TESTING_VERSION" ] && init_msg "$ERR_CON"

# get newest version of script every time
rm -f ${TMP}/install-vivaldi.sh
wget -q https://downloads.vivaldi.com/snapshot/install-vivaldi.sh -P $TMP && chmod 755 ${TMP}/install-vivaldi.sh

kill -9 $apid

STABLE=''
SNAP=''

# common
BGUI=''
for CUR in vivaldi-stable vivaldi-snapshot
do
	case "$CUR" in
		vivaldi-stable)VER="$CUR_STABLE_VERSION";NVER="$NEW_STABLE_VERSION";;
		vivaldi-snapshot)VER="$CUR_TEST_VERSION";NVER="$NEW_TESTING_VERSION";;
	esac
	# check sfs exists in /mnt/home and not loaded
	XVER=''
	case "$CUR" in
		vivaldi-stable)[ -n "${VIVALDI_STABLE_REVISION}" ] && XVER="-${VIVALDI_STABLE_REVISION}"
		STABLE=${CUR}_${NVER}${XVER}_${CARCH}
		SSFS=${CUR}-${NVER}${XVER}-${arch}
		echo $SSFS;;
		vivaldi-snapshot)[ -n "${VIVALDI_TESTING_REVISION}" ] && XVER="-${VIVALDI_TESTING_REVISION}"
		SNAP=${CUR}_${NVER}${XVER}_${CARCH}
		TSFS=${CUR}-${NVER}${XVER}-${arch}
		echo $TSFS;;
	esac
	# this stops the program on the first match
	check_sfs_exists ${CUR}-${NVER}${XVER}-${arch}~root.sfs
	check_sfs_exists ${CUR}-${NVER}${XVER}-${arch}~spot.sfs
	# check latest installed
	echo "$VER" "$NVER"
	if [ "$VER" = "$NVER" ];then
		case "$CUR" in
			vivaldi-stable)VIVALDI=false;;
			vivaldi-snapshot)SVIVALDI=false;;
		esac
		continue
	fi
	case "$CUR" in
		vivaldi-stable) [ -n "$VER" ] && VC=false || VC=true;;
		vivaldi-snapshot) [ -n "$VER" ] && SC=false || SC=true;;
	esac
	[ -z "$VER" ] && continue 
	#check if sfs is installed
	is_sfs_loaded $CUR
	if [ $? -eq 0 ];then
		ICON=$SFSICON
		ACT=unload
		PROG=sfs_load
	fi
	# add to main gui
	TBASE1=$(gettext ', an older version is installed. Uninstall it with ')
	TBASE2=$(gettext ' - button on left - and run this program again later.')
	BGUI="$BGUI"'<hbox space-expand="true" space-fill="true">
			<button space-expand="false" space-fill="true">
			  <input file>'$ICON'</input>
			  <width>32</width><height>32</height>
			  <action function="exit">'$ACT'</action>
			</button>
			<text use-markup="true" space-expand="true" space-fill="false" justify="2">
			  <label>"<i>'${CUR}-${VER}${TBASE1}${PROG}${TBASE2}'</i>"</label>
			</text>
		</hbox>'
		echo $VIVALDI $SVIVALDI
done

[ -n "$BGUI" ] && BGUI='<vbox space-expand="true" space-fill="false">'$BGUI'</vbox>'
####
echo ${STABLE}
echo ${SNAP}
STABLEVER=`echo ${SSFS}|grep -oE '[0-9].*\-[a-z]'`
SNAPVER=`echo ${TSFS}|grep -oE '[0-9].*\-[a-z]'`
echo ${STABLEVER%\-*}
echo ${SNAPVER%\-*}

vtt=$(gettext 'is already installed')
tstvtt=$(gettext 'is older than stable so is unavailable')

VEST="$vtt"
VETT="$vtt"

if [ "$VIVALDI" = 'true' ]; then
	VC='true'
	VEST=''
else
	VC='false'
fi
VI='<button image-position="top" theme-icon-size="32" tooltip-text="version '$NEW_STABLE_VERSION' '$VEST'">
			<label>Stable</label>
			<variable>btnvi</variable>
			<input file>/usr/share/pixmaps/vivaldi.svg</input>
			<sensitive>'$VC'</sensitive>
		</button>'
if [ "$SVIVALDI" = 'true' ]; then
	if vercmp ${STABLEVER%\-*} ge ${SNAPVER%\-*} ; then
		SC='false'
		VETT="$tstvtt"
	else
		SC='true'
		VETT=''
	fi
else
	SC='false'
fi
SI='<button image-position="top" theme-icon-size="32" tooltip-text="version '$NEW_TESTING_VERSION' '$VETT'">
			<label>Testing</label>
			<variable>btnSvi</variable>
			<input file>/usr/share/pixmaps/vivaldi-snapshot.svg</input>
			<sensitive>'$SC'</sensitive>
		</button>'

# gui
export DLG='<window title="Install Vivaldi" icon-name="internet" resizable="false">
	<vbox width-request="500" space-expand="false" space-fill="false">
	'"`/usr/lib/gtkdialog/xml_info fixed /usr/share/pixmaps/vivaldi.svg 60 "$MSG1"`"' 
	  <frame>
	  <text><label>'$MSG3'</label></text>
	  <hbox homogeneous="true">
	      <pixmap>
	        <width>32</width>
			<height>32</height>
	        <input file>/usr/share/pixmaps/root.png</input>
	      </pixmap>		  
	      <radiobutton>
			<label>root</label>
			<variable>RADIOBUTTON1</variable>
		  </radiobutton>
		  <pixmap>
	        <width>32</width>
			<height>32</height>
	        <input file>/usr/share/pixmaps/spot.png</input>
	      </pixmap>
	      <radiobutton>
			<label>spot</label>
			<variable>RADIOBUTTON2</variable>
		  </radiobutton>
	  </hbox>
	  <hbox homogeneous="true">
		<checkbox>
			<label>'$MEDIA'</label>
			<variable>CHECK0</variable>
			<default>true</default>
		</checkbox>
	  </hbox>
	  </frame>
	  <frame>
	  '$BGUI'
	  <hbox space-expand="true" space-fill="false">
	    <text><label>'"$MSG2"'</label></text>
	  </hbox>
	  <hbox  space-expand="true" space-fill="false">  
	  '$VI'
	  '$SI'
	  </hbox>
	  </frame>

	  <hbox  space-expand="false" space-fill="false">
	    <button cancel></button>
	  </hbox>
	</vbox>
</window>'

. /usr/lib/gtkdialog/xml_info gtk
case $1 in
 -d)echo "$DLG";;
 *)eval $(gtkdialog -p DLG --styles=/tmp/gtkrc_xml_info.css);;
esac

echo $EXIT $RADIOBUTTON1 $RADIOBUTTON2 $CHECK0
case $EXIT in
	unload)unload_sfs
	exec $0;;
	Stable)
		VIVALDI_STREAM='vivaldi-stable'
		VIVALDI_STREAM_SHORT='vivaldi'
		ZVER=$GET_STABLE_VERSION
		SFS=$SSFS
		func_dld 0;;
	Testing)	
		VIVALDI_STREAM='vivaldi-snapshot'
		VIVALDI_STREAM_SHORT='vivaldi-snapshot'
		SFS=$TSFS
		ZVER=$GET_TESTING_VERSION
		func_dld 1;;
	*)exit;;
esac
[ "$RADIOBUTTON1" == 'true' ] && EXEC="${VIVALDI_STREAM} --no-sandbox" TAG='~root' \
 || EXEC="run-as-spot ${VIVALDI_STREAM}" TAG='~spot'

BSFS=${SFS}${TAG}.sfs
# errors more critical here on in
gtkdialog-splash -close never -bg CornflowerBlue -text "$WMSG" &
export wpid=$!
set -e
cd $TMP # here we extract and create sfs
func_extract ${VIVALDI_STREAM}_${ZVER}_${CARCH}.deb
(
	cd $TMPDIR
	# Symlink desktop environment icons - this block of code thanks to ruario
	for png in opt/$VIVALDI_STREAM_SHORT/product_logo_*.png; do
		pngsize="${png##*/product_logo_}"
		mkdir -p usr/share/icons/hicolor/${pngsize%.png}x${pngsize%.png}/apps
		(
			cd usr/share/icons/hicolor/${pngsize%.png}x${pngsize%.png}/apps/
			ln -s ../../../../../../opt/$VIVALDI_STREAM_SHORT/product_logo_${pngsize} $VIVALDI_STREAM.png
		)
	done

	# fix.desktop
	sed -i -e "s/Icon=.*/Icon=${VIVALDI_STREAM}.svg/" \
			-e "s/Exec.*/Exec=$EXEC /" \
			-e 's/Categories=.*/Categories=WebBrowser;/' \
			usr/share/applications/*.desktop
	
	# Correct the Vivaldi sandbox permissions
	chmod 4755 opt/$VIVALDI_STREAM_SHORT/vivaldi-sandbox
)
kill -9 $wpid
if [ "$CHECK0" = 'true' ] ; then 
	install_prop_media
fi
func_mk_sfs "$BSFS"
cd -
# make sure sfs package exists and load it
if [ -e ${TMP}/${BSFS} ] ; then
	init_msg "${BSFS} $OKSFS1 ${TMP} $OKSFS2" yes
	load_sfs ${TMP}/${BSFS} || init_msg "$ESFS ${TMP}/${BSFS}"
fi
# announce success
	if [ -f /tmp/sfs_load/loop_device_list ] && grep -q "${BSFS}" /tmp/sfs_load/loop_device_list ; then
		init_msg "$VIVALDI_STREAM $SMSG" yes
	else
		init_msg "$VIVALDI_STREAM $SMSG_NOINST $TMP" yes
	fi
# END
